from shortcrust.gl2 import *
from shortcrust.buffer import AttributeBuffer
from shortcrust.shader import ShaderProgram


class FaderShader(ShaderProgram):
	vertex_shader = """
		attribute vec3 aVertexPosition;

		void main(void) {
			gl_Position = vec4(aVertexPosition, 1.0);
		}
	"""

	fragment_shader = """
#ifdef GLES2
		precision mediump float;
#endif

		uniform float uAlpha;

		void main(void) {
			gl_FragColor = vec4(
				vec3(1.0, 1.0, 1.0),
				uAlpha);
		}
	"""

	def __init__(self):
		super(FaderShader, self).__init__()
		self.aVertexPosition = self.get_attrib('aVertexPosition')
		self.uAlpha = self.get_uniform('uAlpha')

	def draw_mesh(self, mesh, alpha):
		mesh.positions.attach(self.aVertexPosition)
		glUniform1f(self.uAlpha, alpha)
		mesh.draw()


# copy and paste FTW.
class BlackFaderShader(ShaderProgram):
	vertex_shader = """
		attribute vec3 aVertexPosition;

		void main(void) {
			gl_Position = vec4(aVertexPosition, 1.0);
		}
	"""

	fragment_shader = """
#ifdef GLES2
		precision mediump float;
#endif

		uniform float uAlpha;

		void main(void) {
			gl_FragColor = vec4(
				vec3(0.0, 0.0, 0.0),
				uAlpha);
		}
	"""

	def __init__(self):
		super(BlackFaderShader, self).__init__()
		self.aVertexPosition = self.get_attrib('aVertexPosition')
		self.uAlpha = self.get_uniform('uAlpha')

	def draw_mesh(self, mesh, alpha):
		mesh.positions.attach(self.aVertexPosition)
		glUniform1f(self.uAlpha, alpha)
		mesh.draw()


class FaderMesh(object):
	mode = GL_TRIANGLE_STRIP

	def __init__(self):
		self.positions = AttributeBuffer([
			(-1, -1, -0.999), (-1, 1, -0.999), (1, -1, -0.999), (1, 1, -0.999)
		])
		self.vertex_count = self.positions.element_count

		self.texture_coords = AttributeBuffer([
			(0.0, 0.75), (0.0, 0.0), (1.0, 0.75), (1, 0.0)
		])

	def draw(self):
		glDrawArrays(self.mode, 0, self.vertex_count)
