from shortcrust.shader import ShaderProgram
from shortcrust.gl2 import *


class TextureShader(ShaderProgram):
	vertex_shader = """
		attribute vec3 aVertexPosition;
		attribute vec3 aVertexNormal;
		attribute vec2 aTexturePosition;

		uniform mat4 uMVMatrix;
		uniform mat4 uPMatrix;

		varying vec2 vTextureCoord;

		void main(void) {
			/* apply MVMatrix to move vertex into correct position within the scene */
			vec4 position = uMVMatrix * vec4(aVertexPosition, 1.0);

			/* project vertex into screen space */
			gl_Position = uPMatrix * position;

			/* pass texture coordinates through to fragment shader for interpolation */
			vTextureCoord = aTexturePosition;
		}
	"""

	fragment_shader = """
#ifdef GLES2
		precision mediump float;
#endif

		varying vec2 vTextureCoord;

		uniform sampler2D uSampler;

		void main(void) {
			vec3 materialColor = texture2D(uSampler, vTextureCoord).rgb;

			gl_FragColor = vec4(materialColor, 1.0);
		}
	"""

	def __init__(self):
		super(TextureShader, self).__init__()
		self.use()

		self.aVertexPosition = self.get_attrib('aVertexPosition')
		self.aTexturePosition = self.get_attrib('aTexturePosition')

		self.uPMatrix = self.get_uniform('uPMatrix')
		self.uMVMatrix = self.get_uniform('uMVMatrix')
		self.uSampler = self.get_uniform('uSampler')

	def _set_pMatrix(self, pMatrix):
		glUniformMatrix4fv(self.uPMatrix, 1, GL_FALSE, pMatrix)

	pMatrix = property(fset=_set_pMatrix)

	def _set_mvMatrix(self, mvMatrix):
		glUniformMatrix4fv(self.uMVMatrix, 1, GL_FALSE, mvMatrix)

	mvMatrix = property(fset=_set_mvMatrix)

	def draw_mesh(self, mesh):
		mesh.texture.activate(GL_TEXTURE0)
		glUniform1i(self.uSampler, 0)
		mesh.positions.attach(self.aVertexPosition)
		mesh.texture_positions.attach(self.aTexturePosition)
		mesh.draw()
